/*
    SDP Editor
    Copyright 2006 Martin Storsj

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    Martin Storsj
    martin@martin.st
*/

#include "servicesearcher.h"
#include <bt_sock.h>
#include <flogger.h>
/* Heavily influenced by Nokia's examples */

enum TCheckType { ECheckType, ECheckValue, EReadValue, ECheckEnd, ECheckFinish };

static const struct {
	TCheckType iCheck;
	TSdpElementType iType;
	TInt iValue;
} KSdpRecord[] = {
	{ ECheckType, ETypeDES },
		{ ECheckType, ETypeDES },
			{ ECheckValue, ETypeUUID, KL2CAP },
		{ ECheckEnd },
		{ ECheckType, ETypeDES },
			{ ECheckValue, ETypeUUID, KRFCOMM },
			{ EReadValue, ETypeUint },
		{ ECheckEnd },
		{ ECheckType, ETypeDES },
			{ ECheckValue, ETypeUUID, KOBEX },
		{ ECheckEnd },
	{ ECheckEnd },
	{ ECheckFinish }
};
	
CServiceSearcher::CServiceSearcher() {
}

CServiceSearcher::~CServiceSearcher() {
	delete iAgent;
	delete iSdpSearchPattern;
}

void CServiceSearcher::FindServiceL(TUUID& aUuid, TBTDevAddr& aAddr, TRequestStatus& aStatus) {
	delete iAgent;
	iAgent = NULL;
	delete iSdpSearchPattern;
	iSdpSearchPattern = NULL;

	iUuid = aUuid;
	iServiceFindStatus = &aStatus;
	*iServiceFindStatus = KRequestPending;
	iAgent = CSdpAgent::NewL(*this, aAddr);
	iSdpSearchPattern = CSdpSearchPattern::NewL();
	iSdpSearchPattern->AddL(iUuid);
	iAgent->SetRecordFilterL(*iSdpSearchPattern);
	iServiceFound = EFalse;
	iAgent->NextRecordRequestL();
}

void CServiceSearcher::FindServiceChannelL(TUUID& aUuid, TBTDevAddr& aAddr, TRequestStatus& aStatus) {
	iFindType = EFindChannel;
	FindServiceL(aUuid, aAddr, aStatus);
}

void CServiceSearcher::FindServiceHandleL(TUUID& aUuid, TBTDevAddr& aAddr, TRequestStatus& aStatus) {
	iFindType = EFindHandle;
	FindServiceL(aUuid, aAddr, aStatus);
}

void CServiceSearcher::CancelFind() {
	Finished(KErrNone);
}

void CServiceSearcher::AttributeRequestComplete(TSdpServRecordHandle aHandle, TInt aError) {
	TRAPD(err, AttributeRequestCompleteL(aHandle, aError));
	if (err != KErrNone)
		Finished(err);
}

void CServiceSearcher::AttributeRequestResult(TSdpServRecordHandle aHandle, TSdpAttributeID aAttrID, CSdpAttrValue* aAttrValue) {
	TRAPD(err, AttributeRequestResultL(aHandle, aAttrID, aAttrValue));
	if (err != KErrNone)
		Finished(err);
}

void CServiceSearcher::NextRecordRequestComplete(TInt aError, TSdpServRecordHandle aHandle, TInt aTotalRecordsCount) {
	TRAPD(err, NextRecordRequestCompleteL(aError, aHandle, aTotalRecordsCount));
	if (err != KErrNone)
		Finished(err);
}

void CServiceSearcher::AttributeRequestCompleteL(TSdpServRecordHandle aHandle, TInt aError) {
	if (iServiceFound) return;
	iAgent->NextRecordRequestL();
}

void CServiceSearcher::AttributeRequestResultL(TSdpServRecordHandle aHandle, TSdpAttributeID aAttrID, CSdpAttrValue* aAttrValue) {
	if (iServiceFound) return;
	iIndex = 0;
	aAttrValue->AcceptVisitorL(*this);
	iServiceFound = ETrue;
	Finished(KErrNone);
}

void CServiceSearcher::NextRecordRequestCompleteL(TInt aError, TSdpServRecordHandle aHandle, TInt aTotalRecordsCount) {
	if (iServiceFound) return;
	if (aError == KErrEof)
		Finished(KErrNone);
	else if (aError == -6004)
		Finished(KErrTimedOut);
	else if (aError != KErrNone)
		Finished(aError);
	else if (aTotalRecordsCount == 0)
		Finished(KErrNotFound);
	else {
		switch (iFindType) {
		case EFindChannel:
			iAgent->AttributeRequestL(aHandle, KSdpAttrIdProtocolDescriptorList);
			break;
		case EFindHandle:
			iHandle = aHandle;
			iServiceFound = ETrue;
			Finished(KErrNone);
			break;
		}
	}
}

void CServiceSearcher::Finished(TInt aError) {
	if (aError == KErrNone && !iServiceFound)
		aError = KErrNotFound;
	if (*iServiceFindStatus == KRequestPending)
		User::RequestComplete(iServiceFindStatus, aError);
}

TInt CServiceSearcher::GetChannel() {
	return iChannel;
}

TSdpServRecordHandle CServiceSearcher::GetHandle() {
	return iHandle;
}

void CServiceSearcher::VisitAttributeValueL(CSdpAttrValue& aValue, TSdpElementType aType) {
	switch (KSdpRecord[iIndex].iCheck) {
	case ECheckType:
		if (aValue.Type() != KSdpRecord[iIndex].iType)
			User::Leave(KErrGeneral);
		break;
	case ECheckValue:
		if (aValue.Type() != KSdpRecord[iIndex].iType)
			User::Leave(KErrGeneral);
		switch (KSdpRecord[iIndex].iType) {
		case ETypeUUID:
			if (aValue.UUID() != TUUID(KSdpRecord[iIndex].iValue))
				User::Leave(KErrGeneral);
			break;
		case ETypeUint:
			if (aValue.Uint() != (TUint) KSdpRecord[iIndex].iValue)
				User::Leave(KErrGeneral);
			break;
		default:
			break;
		}
		break;
	case EReadValue:
		if (aValue.Type() != KSdpRecord[iIndex].iType)
			User::Leave(KErrGeneral);
		iChannel = aValue.Uint();
		break;
	case ECheckEnd:
	case ECheckFinish:
		User::Leave(KErrGeneral);
	}
	iIndex++;
}

void CServiceSearcher::StartListL(CSdpAttrValueList& aList) {
}

void CServiceSearcher::EndListL() {
	if (KSdpRecord[iIndex].iCheck != ECheckEnd)
		User::Leave(KErrGeneral);
	iIndex++;
}

